# !/usr/bin/python

#####################################
# Lightsensor
# Author: Aron Schneider
#####################################

import RPi.GPIO as GPIO
import time
import log

class Light():

    def __init__(self, krull):

        # ******* SETTINGS ********** #
        self.lightActivation = 0.5 # Seconds of Light till light is activated
        self.darkActivation = 0.5 # Seconds of Light till dark is activated
        self.boredSolve = 3 # seconds till bored solved


        # ******* SYSTEM ********** #
        self.krull = krull
        self.looping = True
        self.lastLight = 0
        self.lastDark = 0
        self.current = 0

        GPIO.setmode(GPIO.BCM)

        if not self.krull.deactivateLight:
            self.light = 0
            self.dark = 1
            self.sensor = 18
            GPIO.setup(self.sensor, GPIO.IN, GPIO.PUD_DOWN)


        self.lightTimer = 0
        self.darkTimer = 0

        self.logtype = "interaction"
        self.logcat = "lightsensor"


    def loop(self):
        log.debug("lightsensor loop started")
        if not self.krull.deactivateLight:
            while self.looping:
                newState = GPIO.input(self.sensor)
                if  newState == self.light:
                    if self.lightTimer == 0:
                        self.lightTimer = time.time()
                        log.log(self.logtype, self.logcat, "lighttimer start")
                    else:
                        t = time.time() - self.lightTimer
                        if t >= self.lightActivation:
                            self.lastLight = time.time()
                            self.darkTimer = 0
                            self.current = self.light
                else:
                    if self.darkTimer == 0:
                        self.darkTimer = time.time()
                        log.log(self.logtype, self.logcat, "darktimer start")
                    else:
                        t = time.time() - self.lightTimer
                        if t >= self.darkActivation:
                            self.lastDark = time.time()
                            self.lightTimer = 0
                            self.current = self.dark
                time.sleep(0.1)

