# !/usr/bin/python

#####################################
# Microphone listen to Audio-Input
# Author: Aron Schneider
#####################################

import pyaudio
import numpy as np
import time
import log

class Microphone():

    def __init__(self, krull):
        # ******* SETTINGS ********** #
        self.noiseLevel = 15000
        self.singLevel = 12000
        self.shoutLevel = 30000


        self.noiseActivation = 0.5 # activation of noise mode (filter short silences) after x seconds
        self.quietActivation = 1.5 # activation of quiet mode (filter short level peaks) after x seconds
        self.singDuration = 5 # recognize as singing after x seconds
        self.shoutDuration = 0.3  # recognize input as shouting after x seconds

        self.logtype = "interaction"
        self.logcat = "microphone"

        # ******* SYSTEM ********** #
        self.krull = krull

        self.lastNoise = 0

        if not self.krull.deactivateMic:
            self.chunk_size = 8192
            self.chunk_size = 32768
            self.audio_format = pyaudio.paInt16
            self.sample_rate = 16000

            self.noiseTimer = 0
            self.quietTimer = 0
            self.singTimer = 0
            self.shoutTimer = 0


        self.looping = False

    def stop(self):
        self.stream.stop_stream()
        self.stream.close()
        self.p.terminate()

    def start(self):
        pass

    def loop(self):
        if not self.krull.deactivateMic:
            log.debug("mic read audio input started")
            self.p = pyaudio.PyAudio()
            self.stream = self.p.open(format= self.audio_format, channels=1, rate=self.sample_rate, input=True, frames_per_buffer=self.chunk_size)
            try:
                while self.looping:
                    # grab audio and timestamp
                    audio = np.fromstring(self.stream.read(self.chunk_size), np.int16)
                    level = np.abs(audio).max()
                    log.debug("mic input level: "+str(level))
                    self.process_audio(level)
                    time.sleep(0.01)

            except Exception as e:
                log.error(e)
                self.krull.stopMic()
            except KeyboardInterrupt:
                pass
            finally:
                self.stop()

    def process_audio(self, level):
        if not self.krull.deactivateMic:

            # quiet
            if level <= self.noiseLevel:
                if self.quietTimer == 0:
                    self.quietTimer = time.time()
                    log.debug("quiet timer start")
                else:
                    t = time.time() - self.quietTimer
                    if t >= self.quietActivation:
                        self.noiseTimer = 0
                        self.singTimer = 0
                        self.shoutTimer = 0

            # noisy
            else:
                if self.noiseTimer == 0:
                    self.noiseTimer = time.time()
                    log.debug("noise timer start")
                else:
                    t = time.time() - self.noiseTimer
                    if t >= self.noiseActivation:
                        self.quietTimer = 0
                        self.lastNoise = time.time()
                        #self.singCheck(level)
                        self.shoutCheck(level)



    def shoutCheck(self, level):
        if not self.krull.deactivateMic:
            if level >= self.shoutLevel:
                log.debug("shout level")

                if self.shoutTimer == 0:
                    self.shoutTimer = time.time()
                    log.debug("shout timer start")
                else:
                    t = time.time() - self.shoutTimer
                    if t >= self.shoutDuration:
                        log.log(self.logtype, self.logcat, "shouting")
                        self.krull.shout()
                        self.shoutTimer = 0


    def singCheck(self, level):
        if not self.krull.deactivateMic:
            if level >= self.singLevel:
                if self.singTimer == 0:
                    self.singTimer = time.time()
                    log.debug("mic, sing timer start")
                else:
                    t = time.time() - self.singTimer
                    if t >= self.singDuration:
                        log.log(self.logtype, self.logcat, "singing")
                        self.krull.sing()
                        self.singTimer = 0

    def isNoisy(self):
        if not self.krull.deactivateMic:
            t = time.time() - self.noiseTimer
            if t >= self.noiseActivation:
                return True
            else:
                return False
        else:
            return False

