# !/usr/bin/python

#####################################
# Playercontroller and PlayerClass
# Author: Aron Schneider
#####################################

import random
import time
import log

class PlayerController():

    def __init__(self, krull):

        # ******* SETTINGS ********* #
        self.pause = 3 # pause between to register attepmts

        # ******* SYSTEM  ***********#
        self.loginTime = 0
        self.players = dict()
        self.freePlayers = [1,2,3,4]
        self.currentPlayer = 0
        self.krull = krull
        self.loginLock = False

        self.logtype = "gamestate"
        self.logcat = "player"

    def playerLogin(self, uid):

        # CHECK LAST LOGIN-TIME AND, SET TIMER
        if self.loginTime < time.time() - self.pause:
            # Login only in if no other loginprocess is running
            if not self.loginLock:
                self.loginLock = True
                if self.krull.gamestate == "brain" or self.krull.gamestate == "register":
                    self.krull.mouth.speak("login_mouthFull", True)
                    self.krull.delay()

                # only react if it`s not the current player
                if uid != self.currentPlayer:
                    # is a new player requested?
                    if not self.krull.newPlayerRequest:
                        if self.krull.gamestate == "brain":
                            log.log(self.logtype, self.logcat, "login attempt while no new player is requested")
                            self.krull.mouth.speak("login_wrongPlayer")
                    else:
                        # new player
                        if not uid in self.players:
                            log.log(self.logtype, self.logcat, "login new player")
                            self.createNewPlayer(uid)
                            self.currentPlayer = uid
                            self.playWelcome(uid, 1)
                            # set startRegistred to true if in Krull is in Startloop
                            if self.krull.gamestate == "register":
                                self.krull.setGamestate("initBrain")
                            self.krull.newPlayerRequest = False
                        # player already registred
                        else:
                            log.log(self.logtype, self.logcat, "login player "+str(self.players[uid].registerNumber))
                            self.currentPlayer = uid
                            self.playWelcome(uid)
                            self.krull.newPlayerRequest = False
                            log.log(self.logtype, self.logcat, "player "+str(self.players[uid].registerNumber)+ " logged in")
                        log.setPlayer(self.players[uid].registerNumber)

                else:
                    log.log(self.logtype, self.logcat, "same player login attempt")
            self.loginLock = False
            self.loginTime = time.time()

    def createNewPlayer(self, uid):
        voiceNumber = random.choice(self.freePlayers)
        self.freePlayers.remove(voiceNumber)
        playerNr = len(self.players)+1
        self.players[uid] = Player(self.krull, uid, voiceNumber, playerNr)
        log.log(self.logtype, self.logcat, "new player registred. UID: "+str(uid)+" voice: "+str(self.players[uid].voiceNumber))

    def playWelcome(self, uid, contact=0):
        player = self.players[uid]
        if contact == 0:
            contact = player.contact
        if contact > 6:
            contact = random.choice([2,3,4,5,6])
        if self.krull.mainCondition == "normal":
            line = "login_welcomPlayer"+str(player.voiceNumber)+"contact"+str(contact)
        else:
            line = "login_welcomeCondition_"+self.krull.mainCondition
        self.krull.mouth.speak(line)
        self.krull.eyeFeedback()
        player.madeContact()


class Player():
    def __init__(self, krull, rfid, voiceNumber, registerNumber):
        self.contact = 0
        self.krull = krull
        self.rfid = rfid # RFID Tag No
        self.registerNumber = registerNumber # First, second, third... registred player
        self.voiceNumber = voiceNumber # Number player

    def madeContact(self):
        self.contact += 1