# !/usr/bin/python

#####################################
# RFID Controller
# Author: Aron Schneider
#####################################

import MFRC522
import signal
import RPi.GPIO as GPIO
import time
import log

class RFID():

    def __init__(self, krull):

        # ******* SETTINGS ********** #

        self.delay = 1 # Delay between two rfid-register events

        self.logtype = "interaction"
        self.logcat = "rfid"

        # ******* SYSTEM ********** #

        self.tagP1 = 41
        self.tagP2 = 169
        self.tagP3 = 201
        self.tagP4 = 185
        self.tagFood1 = 121
        self.tagFood2 = 249
        self.tagDrink1= 89
        self.tagDrink2 = 57

        self.krull = krull

        self.startTime = 0

        self.MIFAREReader = MFRC522.MFRC522()
        self.continue_reading = True

        self.looping = True

    def rfidInput(self):

        # Scan for cards
        (self.status,TagType) = self.MIFAREReader.MFRC522_Request(self.MIFAREReader.PICC_REQIDL)
        # Get the UID of the card
        (self.status,uid) = self.MIFAREReader.MFRC522_Anticoll()
        # If we have the UID, continue
        if self.status == self.MIFAREReader.MI_OK:
            log.debug("rfid tag read: "+str(uid[0]))
            return self.gate(uid[0])


    def gate(self, uid):
        if uid == self.tagFood1 or uid == self.tagFood2:
            log.log(self.logtype, self.logcat, "eat")
            self.krull.eat(uid)
            return True
        if uid == self.tagDrink1 or uid == self.tagDrink2:
            log.log(self.logtype, self.logcat, "drink")
            self.krull.drink(uid)
            return True
        elif uid == self.tagP1 or uid == self.tagP2 or uid == self.tagP3 or uid == self.tagP4:
            self.krull.playerLogin(uid)
            return True
        else:
            log.debug("unknown rfid tag: "+str(uid)+" registered")
            return False

    def loop(self):

        log.debug("rfid loop started")

        while self.looping:
            tagfound = self.rfidInput()
            if (tagfound == True):
                time.sleep(self.delay)
            else:
                time.sleep(0.5)


    # Capture SIGINT for cleanup when the script is aborted
    def end_read(self,signal,frame):
        global continue_reading
        #print "Ctrl+C captured, ending read."
        continue_reading = False
        GPIO.cleanup()
