# !/usr/bin/python

#####################################
# Scale-Puzzle
# Author: Aron Schneider
#####################################

import sys
import time
import Adafruit_MPR121.MPR121 as MPR121
import random
import log
from threading import *

class Scales():

    def __init__(self, krull):

        # ******* SETTINGS ********** #
        self.difficulty = 3 # number of scales you have to scratch
        self.ledDuration = 0.1 # flash-duration of scale
        self.ledInterval = 0.1 # break betweeen two flashing scales
        self.ledTouchDuration = 1
        self.replayInterval = 150 # repeat sequence after x seconds

        self.logtype = "interaction"
        self.logcat = "scales"

        # ******* SYSTEM ********** #

        self.krull = krull
        #self.led = self.krull
        self.led = krull.led
        self.looping  = True
        self.replayTimer = 0

        if not self.krull.deactivateScale:
            log.debug("scales init")
            self.cap = MPR121.MPR121()
            if not self.cap.begin():
                log.error('Error initializing MPR121.  Check your wiring!')
                sys.exit(1)
            self.last_touched = self.cap.touched()

            self.scales = dict(
                               scale1 = Scale("1", "scale1", "0"),
                               scale2 = Scale("2", "scale2", "1"),
                               #scale3 = Scale("3", "scale3", "2"),
                               scale4 = Scale("4", "scale4", "3"),
                               scale5 = Scale("5", "scale5", "4"),
                               scale6 = Scale("6", "scale6", "5"),
                               #scale7 = Scale("7", "scale7", "6"),
                               scale8 = Scale("8", "scale8", "7")
                            )

            self.state = "idle"
            self.sequence = list()
            self.sequencePlayer = list()
            self.sequenceSolved = list()

            self.playing = False
            self.listening = False

    def checkLoop(self):
        while True:
            current_touched = self.cap.touched()
            for i in range(12):
                pin_bit = 1 << i
                scale = self.getScale(i)
                if scale != "off":
                    # First check if transitioned from not touched to touched.
                    if current_touched & pin_bit and not self.last_touched & pin_bit:
                        log.debug('scale touch on {0}'.format(i))
                        if scale != "off":
                            self.led.queue.put(["on", scale.led])
                        else:
                            log.debug("scale off")
                    # Next check if transitioned from touched to not touched.
                    elif not current_touched & pin_bit and self.last_touched & pin_bit:
                        log.debug('scale released on {0}'.format(i))
                        if scale != "off":
                            self.led.queue.put(["off", scale.led])
            self.last_touched = current_touched
            time.sleep(0.1)

    def playLoop(self):
        while self.playing:
            if not self.sequence:
                factor = 1
            else:
                factor = len(self.sequence)*3
            for s in self.sequence:
                self.led.queue.put(["on", s.led])
            time.sleep(self.ledDuration*factor)
            for s in self.sequence:
                self.led.queue.put(["off", s.led])
            time.sleep(self.ledInterval*factor)

    def listenLoop(self):
        while self.playing:
            current_touched = self.cap.touched()
            for i in range(12):
                pin_bit = 1 << i
                scale = self.getScale(i)
                if scale != "off":
                    # First check if transitioned from not touched to touched.
                    if current_touched & pin_bit and not self.last_touched & pin_bit:
                        #log.debug('scale touch on {0}'.format(i))
                        if scale in self.sequence:
                            log.debug("pressed scale is in sequence")
                            self.sequence.remove(scale)
                            if not self.sequence:
                                log.log(self.logtype, self.logcat, "puzzle solved")
                                self.playing = False
                                self.sequenceSolved[:] = []
                                self.sequence [:] = []
                                self.allOff()
                                self.krull.scaleSolved()
                            else:
                                self.sequenceSolved.append(scale)
                                self.led.queue.put(["on", scale.led])
                                self.krull.scaleTouch()
                    # Next check if transitioned from touched to not touched.
                    elif not current_touched & pin_bit and self.last_touched & pin_bit:
                        #log.debug('scale released on {0}'.format(i))
                        if scale in self.sequenceSolved:
                            log.debug("released scale is in sequence")
                            self.sequenceSolved.remove(scale)
                            self.sequence.append(scale)
            self.last_touched = current_touched
            time.sleep(0.1)

    def startPuzzle(self):
        if not self.krull.deactivateScale and not self.playing:
            self.setPuzzle()
            #self.state = "play"
            self.playing = "true"
            self.threadPlay = Thread(target=self.playLoop)
            self.threadPlay.setDaemon(True)
            self.threadPlay.start()
            self.threadListen = Thread(target=self.listenLoop)
            self.threadListen.setDaemon(True)
            self.threadListen.start()

    def setPuzzle(self):
        if not self.krull.deactivateScale:
            self.sequence = random.sample(self.scales.values(), self.difficulty)


    def allOff(self):
        if not self.krull.deactivateScale:
            for s in self.scales:
                self.led.queue.put(["off", self.scales[s].led])


    def getScale(self, touchNr):
        if not self.krull.deactivateScale:
            scaleName = 'scale{0}'.format(touchNr+1)
            if scaleName in self.scales:
                return self.scales[scaleName]
            else:
                return "off"


class Scale():

    def __init__(self, name, led, touch):
        self.name = name
        self.led = led
        self.touch = touch


