# !/usr/bin/python

#####################################
# States
# Author: Aron Schneider
#####################################

import random
import math
from threading import *
import time
import log

class State():

    def __init__(self, krull, name):

        # *********** SETTINGS ************#
        self.levelfactor = 3
        self.raiseStartChance = 10 # Percent chance to raise sublevel
        self.chanceRaiseAmount = 1 # increase raiseChance every update by x

        # ******* SYSTEM ********** #
        self.krull = krull
        self.name = name
        self.starttime = time.time()
        self.active = False # condition has to be activated by input
        self.level = 0 # 0 = love, peace and harmony, 1 = trivial, 2 = severe, 3 = emergency
        self.sublevel = 0
        self.raiseChance = self.raiseStartChance
        self.levelTimer = 0
        self.activeStart = 0

        self.logtype = "condition"

    def update(self):
        if self.active:
            if self.level < 3:
                self.raiseLevel()
                self.increaseChance()


    def raiseLevel(self):
        roll = random.randrange(0, 100)
        # print "Roll: "+str(roll)+" Chance: "+str(self.raiseChance)
        add = time.time()-self.starttime
        if random.randrange(0, 100) < self.raiseChance+add/8:
            self.sublevel += 1
            self.raiseChance = self.raiseStartChance+add/30
            #log.log(self.logtype, self.name, "sublevel"+ str(self.sublevel))
            self.setLevel()

    def increaseChance(self):
        self.raiseChance += self.chanceRaiseAmount

    def solve(self):
        self.sublevel = 0
        self.setLevel()
        log.log(self.logtype, self.name, " solved")


    def setLevel(self):
        oldLevel = self.level
        self.level = int(math.floor(self.sublevel / self.levelfactor))
        if self.level > 3:
            self.level = 3
        if oldLevel == 0 and self.level == 1:
            log.debug(self.logtype+" "+self.name+ " leveltimer started")
            self.levelTimer = time.time()
        elif self.level == 0:
            self.levelTimer = 0
        if self.level != oldLevel:
            log.log(self.logtype, self.name, "level"+str(self.level))

    def getEmergency(self):
        if self.level == 3:
            return True
        else:
            return False




class StateInactive(State):

    def __init__(self, krull, name):

        State.__init__(self, krull, name)

        # *********** SETTINGS ************#
        self.activationStartChance = 30 # %-Chance to activate if triggered
        self.activationChanceSteps = 15 # Increase chance if it`s triggered by x

        self.autoDeactivate = 60 # auto-solve condition if its not solved for x seconds

        # ******* SYSTEM ********** #
        self.activationChance = self.activationStartChance

    def update(self):
        if self.active:
            self.checkDeactivate()
        State.update(self)

    def trigger(self):
        log.log(self.logtype, self.name, "triggered")
        if not self.active:
            if random.randrange(0, 100) < self.activationChance:
                self.active = True
                self.activeStart = time.time()
                self.activationChance = self.activationStartChance
                log.log(self.logtype, self.name, "activated")
            else:
                self.activationChance += self.activationChanceSteps
                log.debug(self.logtype+" "+self.name+" triggerchance raised to: "+str(self.activationChance))
        else:
            log.debug(self.logtype+" "+ self.name+ " triggered, but already active")

    def solve(self):
        self.active = False
        State.solve(self)

    def checkDeactivate(self):
        t = time.time() - self.activeStart
        if t >= self.autoDeactivate:
            self.active = False
            self.sublevel = 0
            self.setLevel()
            if self.name == "headache" or self.name == "hickup":
                self.krull.stopMic()
            log.log(self.logtype, self.name, "auto-solved")


class StateBored(StateInactive):

    def __init__(self, krull, name):

        # *********** SETTINGS ************#
        self.stillTime = 20 # seconds Krull have to be held still to trigger bored
        self.silentTime = 20 # seconds of silence around Krull to trigger bored
        self.lightTime = 10 # seconds of light around Krull to solve bored
        self.triggerInterval = 10 # if conditions are true, trigger every x second

        # ******* SYSTEM ********** #
        StateInactive.__init__(self, krull, name)

        self.triggerTimer = 0
        self.task = "none"
        self.looping = True

    def loop(self):
        log.debug(self.logtype+" "+self.name+" activation loop started")
        while self.looping:
            if not self.active:
                tMoved = time.time() - self.krull.gyro.lastMoved
                tNoise = time.time() - self.krull.mic.lastNoise
                if ((self.krull.gyro.lastMoved > 0) and (tMoved > self.stillTime)) or ((self.krull.mic.lastNoise > 0) and (tNoise > self.silentTime)):
                    if self.triggerTimer == 0:
                        self.triggerTimer = time.time()
                        log.debug(self.logtype+" "+self.name+" trigger timer start")
                    else:
                        t = time.time() - self.triggerTimer
                        if t >= self.triggerInterval:
                            StateInactive.trigger(self)
                            if self.active:
                                self.setTask()
                            self.triggerTimer = 0
            time.sleep(1)

    def setTask(self):
        if not self.krull.deactivateLight and self.krull.light.current == self.krull.light.dark:
            self.task = "light"
            self.looping = True
            self.threadSolve= Thread(target=self.solveLight)
            self.threadSolve.setDaemon(True)
            self.threadSolve.start()
        else:
            if not self.krull.deactivateMic and not self.krull.deactivateLight:
                self.task = random.choice(["sing", "throw", "any"])
            else:
                self.task = random.choice(["throw"])
        log.log(self.logtype, self.name, "task set to: "+self.task)
        if (self.task == "sing"):
            self.krull.startMic()

    def solveLight(self):
        log.debug(self.logtype+" "+self.name+" light solve loop started")
        while self.looping:
            if time.time() - self.krull.light.lastDark > self.lightTime:
                StateInactive.solve(self)
                self.krull.boredLightSolved()
                self.looping = False
            time.sleep(1)

    def solve(self, solution):
        if (self.task == "any" and solution != "light") or self.task == solution:
            StateInactive.solve(self)
            self.krull.boredSolved()


class StateInactiveActiveSolve(StateInactive):

     def __init__(self, krull, name):
         StateInactive.__init__(self, krull, name)
         self.solveloop = False

     def trigger(self):
        StateInactive.trigger(self)
        if self.active:
            log.debug(self.logtype+" "+self.name+" solve loop started")
            self.solveloop = True
            self.threadSolve= Thread(target=self.solve)
            self.threadSolve.setDaemon(True)
            self.threadSolve.start()

     def solve(self):
         self.solveloop = False
         StateInactive.solve(self)


class StateHeadache(StateInactiveActiveSolve):

    def __init__(self, krull, name):

        # *********** SETTINGS ************#
        self.stillTime = 10 # seconds Krull have to be held still to solve condition
        self.darkTime = 10 # seconds Krull have to be in darkness to solve condition
        self.activationStartChance = 10 # %-Chance to activate if triggered
        self.activationChanceSteps = 5 # Increase chance if it`s triggered by x

        # ******* SYSTEM ********** #
        StateInactiveActiveSolve.__init__(self, krull, name)
        self.activationChance = self.activationStartChance

    def trigger(self):
        StateInactiveActiveSolve.trigger(self)
        if self.active:
            self.krull.startMic()

    def solve(self):

        while self.solveloop:

            if not self.krull.deactivateLight:
                if (time.time() - self.krull.mic.lastNoise > self.stillTime) and (time.time() - self.krull.light.lastLight > self.darkTime):
                    StateInactiveActiveSolve.solve(self)
                    self.krull.headacheSolved()
                    self.solveloop = False
            else:
                if (time.time() - self.krull.mic.lastNoise> self.stillTime):
                    StateInactiveActiveSolve.solve(self)
                    self.krull.headacheSolved()
                    self.solveloop = False

            time.sleep(0.5)


class StateSick(StateInactiveActiveSolve):

    def __init__(self, krull, name):

        # *********** SETTINGS ************#
        self.stillTime = 10 # seconds Krull have to be held still to solve condition

        # ******* SYSTEM ********** #
        StateInactiveActiveSolve.__init__(self, krull, name)

    def solve(self):
        while self.solveloop:
            if time.time() - self.krull.gyro.lastMoved > self.stillTime:
                self.krull.sickSolved()
                StateInactiveActiveSolve.solve(self)
                self.solveloop = False
            time.sleep(1)



class StateHickup(StateInactive):

     def __init__(self, krull, name):
         # *********** SETTINGS ************#
        self.activationStartChance = 100 # %-Chance to activate if triggered
        self.activationChanceSteps = 50 # Increase chance if it`s triggered by x

        # ******* SYSTEM ********** #
        StateInactive.__init__(self, krull, name)
        self.activationChance = self.activationStartChance

     def trigger(self):

         if not self.krull.deactivateMic:
            StateInactive.trigger(self)
            if self.active:
                log.debug("mic active")
                self.krull.startMic()


class StatePain(StateInactive):

     def __init__(self, krull, name):
        StateInactive.__init__(self, krull, name)
        self.side = 0

        # *********** SETTINGS ************#
        self.activationStartChance = 20 # %-Chance to activate if triggered
        self.activationChanceSteps = 1 # Increase chance if it`s triggered by x

        # *********** SYSTEM ************#
        self.activationChance = self.activationStartChance

     def trigger(self):
         StateInactive.trigger(self)
         if self.active:
             self.setSide()

     def setSide(self):
        self.side = self.krull.gyro.getRandomSide()
        # check if target side == current side --> reset
        if (self.side == self.krull.gyro.currentSide):
            self.setSide()
        log.log(self.logtype, self.name, "side set to: "+str(self.side))



class StateScale(StateInactive):

    def __init__(self, krull, name):
        StateInactive.__init__(self, krull, name)

         # *********** SETTINGS ************#
        self.activationStartChance = 20 # %-Chance to activate if triggered
        self.activationChanceSteps = 1 # Increase chance if it`s triggered by x

        # *********** SYSTEM ************#
        self.activationChance = self.activationStartChance

    def trigger(self):
        if not self.krull.deactivateScale:
             StateInactive.trigger(self)
             if self.active:
                 self.activatePuzzle()

    def activatePuzzle(self):
        self.krull.scale.startPuzzle()


class StateToothache(StateInactive):

    def __init__(self, krull, name):
        StateInactive.__init__(self, krull, name)

        # *********** SETTINGS ************#
        self.activationStartChance = 50 # %-Chance to activate if triggered
        self.activationChanceSteps = 25 # Increase chance if it`s triggered by x

        # *********** SYSTEM ************#
        self.activationChance = self.activationStartChance

    def trigger(self):
        if not self.krull.deactivateTeeth:
             StateInactive.trigger(self)
             if self.active:
                 self.activatePuzzle()

    def activatePuzzle(self):
        self.krull.teeth.setPuzzle()


class StateActive(State):

    def __init__(self, krull, name):

        State.__init__(self, krull, name)

        # *********** SETTINGS ************#
        self.startMin = 7
        self.startMax = 12
        self.levelfactor = 10

        # ******* SYSTEM ********** #
        l = random.randrange(self.startMin, self.startMax)
        self.sublevel = l
        self.active = True
        self.setLevel()

