# !/usr/bin/python

#####################################
# Teeth-Puzzle
# Author: Aron Schneider
#####################################

import time
import RPi.GPIO as GPIO
import random
import log

class Teeth():

    def __init__(self, krull):

        # ******* SETTINGS ********** #
        self.maxSteps = 5 # Max steps it takes to solve the game
        self.restartDelay = 5  # Reboot game if all 4 teeth are held down for x seconds

        self.logtype = "interaction"
        self.logcat = "teeth"


        # ******* SYSTEM ********** #
        self.krull = krull
        self.led = krull.led
        self.looping = True

        GPIO.setmode(GPIO.BCM)

        if not self.krull.deactivateTeeth:


            # init buttons
            self.button1 = 4
            self.button2 = 17
            self.button3 = 27
            self.button4 = 22
            self.buttons = [self.button1, self.button2 , self.button3 , self.button4]
            for b in self.buttons :
                GPIO.setup(b, GPIO.IN, GPIO.PUD_DOWN)

            # init LEDs
            self.led1 = "tooth1"
            self.led2 = "tooth2"
            self.led3 = "tooth3"
            self.led4 = "tooth4"
            self.down = True

            self.leds = [self.led1, self.led2 , self.led3 , self.led4]
            time.sleep(0.1)
            self.teeth = dict(
                              tooth1 = Tooth("1", self.led1, self.button1),
                              tooth2 = Tooth("2", self.led2, self.button2),
                              #tooth3 = Tooth("3", self.led3, self.button3),
                              tooth3 = Tooth("3", self.led4, self.button4)
                            )

            self.toothache = False
            self.counter = 0
            self.restartTimer = 0
            self.puzzle = False



    def setPuzzle(self):
        if not self.krull.deactivateTeeth:
            self.toothache = True
            log.log(self.logtype, self.logcat, "start teeth puzzle")
            randomTooth = random.choice(self.teeth.values())
            self.activateTooth(randomTooth)
            self.counter = 0

    def activateTooth(self, t):
        log.log(self.logtype, self.logcat, "pain in :"+t.name)
        t.pain = True
        log.debug("random tooth: "+t.ledx)
        self.led.queue.put(["on", t.ledx])


    def deativateTooth(self, t):
        log.log(self.logtype, self.logcat, "no more pain in :"+t.name)
        t.pain = False
        self.led.queue.put(["off", t.ledx])

    def spreadPain(self, tooth):
        self.deativateTooth((tooth))
        # random left spread
        current = int(tooth.name)
        if (self.counter >= self.maxSteps-2):
            if current <=2:
                next = current-1
            else:
                next = current+1
        else:
            next = random.choice([current-1, current+1])
        if next >= 1 and next <= 3:
            self.krull.teethTouch()
            self.activateTooth(self.teeth["tooth"+str(next)])
            self.counter += 1
        else:
            log.log(self.logtype, self.logcat, "puzzle solved")
            self.krull.teethSolved()

    def reset(self):
        log.debug("teeth reset start")
        self.toothache = False
        self.counter = 0
        self.restartTimer = 0
        for t in self.teeth:
            self.led.queue.put(["on", self.teeth[t].getLed()])
            #self.matrix.off(self.teeth[t].led)
        log.debug("teeth reset end")

    def loop(self):
        log.debug("teeth loop started")
        try:
            while self.looping:
                self.checkRestart()
                #log.debug("teeth loop running")
                if not self.krull.deactivateTeeth and self.puzzle:
                    # Original
                    if self.toothache:
                        #check for button input
                        for t in self.teeth.itervalues():
                            if t.pain:
                                if GPIO.input(t.button) == self.down:
                                    self.spreadPain((t))
                                    log.log(self.logtype, self.logcat,  "tooth held down: "+t.name)
                        time.sleep(0.01)
                    else:
                        time.sleep(0.5)
                else:
                    time.sleep(0.5)
        except KeyboardInterrupt:
            pass
        finally:
            pass

    def checkRestart(self):
        allDown = True
        for t in self.teeth.itervalues():
            #log.debug("tooth "+t.name+" up")
            if GPIO.input(t.button) != self.down:
                allDown = False
            else:
                log.debug("tooth "+t.name+" down")

        if allDown:
            #log.debug("4 teeth held down")
            if self.restartTimer > 0:
                t = time.time() - self.restartTimer
                log.debug("4 teeth held down for: "+str(t) +" seconds")
                if t >= self.restartDelay:
                    self.restartTimer = 0
                    self.krull.setGamestate("restart")
                    time.sleep(3)
            else:
                log.debug("start restart timer")
                self.restartTimer = time.time()
        else:
            self.restartTimer = 0


class Tooth():
    def __init__(self, name, l, button):
        #log.debug("init tooth. name: "+str(name)+" led: "+str(l)+" button: "+str(button))

        self.name = name
        self.ledx = l
        self.button = button
        self.pain = False
        #log.debug("init tooth. name: "+str(name)+" ledx: "+str(self.ledx)+" button: "+str(button))

    def down(self):
        if GPIO.input(self.button) == self.down:
            return True
        else:
            return False

    def getLed(self):
        #log.debug("return led: "+self.ledx)
        return self.ledx
